<?php

namespace Drupal\liveblog\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\liveblog\LiveblogRendererInterface;
use Drupal\liveblog\Utility\Payload;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller providing the resource for the liveblog posts list.
 */
class LiveblogListController extends ControllerBase {

  /**
   * Renders content in a special way.
   *
   * @var \Drupal\liveblog\LiveblogRendererInterface
   */
  protected $liveblogRenderer;

  /**
   * Manages entity type plugin definitions.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Turns a render array into an HTML string.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Forward-compatibility shim for Symfony's RequestStack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * Construct a LiveblogController object.
   *
   * @param \Drupal\liveblog\LiveblogRendererInterface $liveblog_renderer
   *   Renders content in a special way.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Manages entity type plugin definitions.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   Turns a render array into an HTML string.
   * @param \Drupal\Core\Http\RequestStack $request_stack
   *   Forward-compatibility shim for Symfony's RequestStack.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(LiveblogRendererInterface $liveblog_renderer, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, RequestStack $request_stack) {
    $this->liveblogRenderer = $liveblog_renderer;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
    $this->request = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('liveblog.renderer'),
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('request_stack')
    );
  }

  /**
   * Returns a liveblog post form wrapped in a json response.
   *
   * @todo Add render caching.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The liveblog node.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response containing the liveblog post form.
   */
  public function getList(Node $node) {
    if ($node->bundle() != 'liveblog') {
      throw new NotFoundHttpException();
    }

    $request = $this->getRequest();
    $items_per_page = (int) $request->get('items_per_page') ?: 10;
    $created_op = $request->get('created_op') == '<' ? $request->get('created_op') : '>';
    $sort_order = $request->get('sort_order') == 'ASC' ? 'ASC' : 'DESC';
    $timestamp = (int) $request->get('created') ?: 0;

    $storage = $this->getEntityTypeManager()->getStorage('liveblog_post');

    $query = $storage->getQuery();
    $query->condition('status', 1);
    $query->condition('created', $timestamp, $created_op);
    $query->condition('liveblog.entity.nid', $node->id());
    $query->sort('created', $sort_order);
    $query->range(0, $items_per_page);
    $ids = $query->accessCheck()->execute();

    $entities = $storage->loadMultiple($ids);

    if (!$entities) {
      return new JsonResponse([]);
    }

    $content = $render_array = [];
    /** @var \Drupal\liveblog\Entity\LiveblogPost[] $entities */
    foreach ($entities as $entity) {
      $result = Payload::create($entity)->getPayload();

      // Collect all the render arrays. Will be used later to get the libraries
      // and commands needed for the frontend.
      $render_array[] = $result['content'];

      // Render each post separately to prepare a list item content.
      $result['content'] = $this->getRenderer()->render($result['content']);
      $content[] = $result;
    }

    // Render all the posts together to get the libraries and commands.
    $result = $this->liveblogRenderer->render($render_array);

    $result['content'] = $content;

    return new JsonResponse($result);
  }

  /**
   * Returns current request.
   *
   * @return \Symfony\Component\HttpFoundation\Request
   *   The currently active request object.
   */
  protected function getRequest() {
    return $this->request->getCurrentRequest();
  }

  /**
   * GEts the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  protected function getEntityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Returns the render API renderer.
   *
   * @return \Drupal\liveblog\LiveblogRenderer
   *   The render API renderer.
   */
  protected function getRenderer() {
    return $this->renderer;
  }

}
